@@ -5,6 +5,7 @@
 #include <QApplication>
 #include <qmath.h>
 #include <QDir>
+#include <QX11Info>
 
 #include <X11/Xlib.h>
 #include <X11/XKBlib.h>
@@ -13,6 +14,11 @@
 #include <X11/extensions/XKBrules.h>
 #include <X11/extensions/XKBstr.h>
 
+static const int XKeyPress = KeyPress;
+static const int XKeyRelease = KeyRelease;
+#undef KeyPress
+#undef KeyRelease
+
 #include <math.h>
 #include <fcitx-config/hotkey.h>
 
@@ -107,6 +113,12 @@ static QString FcitxXkbFindXkbRulesFile()
     return rulesFile;
 }
 
+static KeySym normalizedKeySym(KeySym sym) {
+    if (sym == XK_ISO_Left_Tab) {
+        return XK_Tab;
+    }
+    return sym;
+}
 
 KeyboardLayoutWidget::KeyboardLayoutWidget(QWidget* parent): QWidget(parent),
     ratio(1.0),
@@ -134,28 +146,8 @@ KeyboardLayoutWidget::KeyboardLayoutWidget(QWidget* parent): QWidget(parent),
     l3mod = XkbKeysymToModifiers (QX11Info::display(),
                                   XK_ISO_Level3_Shift);
 
-    XkbSelectEventDetails (QX11Info::display(), XkbUseCoreKbd,
-                   XkbIndicatorStateNotify,
-                   xkb->indicators->phys_indicators,
-                   xkb->indicators->phys_indicators);
-
     xkbOnDisplay = true;
 
-    int mask = (XkbStateNotifyMask | XkbNamesNotifyMask |
-    XkbControlsNotifyMask | XkbIndicatorMapNotifyMask |
-    XkbNewKeyboardNotifyMask);
-    XkbSelectEvents (QX11Info::display(), XkbUseCoreKbd, mask, mask);
-
-
-    mask = XkbGroupStateMask | XkbModifierStateMask;
-    XkbSelectEventDetails (QX11Info::display(), XkbUseCoreKbd,
-                   XkbStateNotify, mask, mask);
-
-    mask = (XkbGroupNamesMask | XkbIndicatorNamesMask);
-    XkbSelectEventDetails (QX11Info::display(), XkbUseCoreKbd,
-                   XkbNamesNotify, mask, mask);
-
-
     alloc ();
     init();
     initColors();
@@ -293,7 +285,7 @@ void KeyboardLayoutWidget::setKeyboard(XkbComponentNamesPtr names)
 {
     release();
     if (xkb)
-        XkbFreeKeyboard(xkb, 0, TRUE);
+        XkbFreeKeyboard(xkb, 0, true);
     if (names) {
         xkb = XkbGetKeyboardByName (QX11Info::display(), XkbUseCoreKbd,
                       names, 0,
@@ -301,8 +293,8 @@ void KeyboardLayoutWidget::setKeyboard(XkbComponentNamesPtr names)
                       XkbGBN_KeyNamesMask |
                       XkbGBN_OtherNamesMask |
                       XkbGBN_ClientSymbolsMask |
-                      XkbGBN_IndicatorMapMask, FALSE);
-        xkbOnDisplay = FALSE;
+                      XkbGBN_IndicatorMapMask, false);
+        xkbOnDisplay = false;
     } else {
         xkb = XkbGetKeyboard (QX11Info::display(),
                            XkbGBN_GeometryMask |
@@ -312,7 +304,7 @@ void KeyboardLayoutWidget::setKeyboard(XkbComponentNamesPtr names)
                            XkbGBN_IndicatorMapMask,
                            XkbUseCoreKbd);
         XkbGetNames (QX11Info::display(), XkbAllNamesMask, xkb);
-        xkbOnDisplay = TRUE;
+        xkbOnDisplay = true;
     }
 
     if (xkb == NULL)
@@ -703,7 +695,9 @@ void KeyboardLayoutWidget::generatePixmap(bool force)
     if (w == image.width() && h == image.height() && !force)
         return;
 
-    image = QImage(w, h, QImage::Format_ARGB32);
+    image = QPixmap(QSize(w, h) * devicePixelRatio());
+    image.setDevicePixelRatio(devicePixelRatio());
+    image.fill(Qt::transparent);
     QPainter painter(&image);
     painter.setRenderHint(QPainter::Antialiasing, true);
     painter.setCompositionMode(QPainter::CompositionMode_Source);
@@ -874,10 +868,10 @@ void KeyboardLayoutWidget::drawPolygon(QPainter* painter, QColor fill_color, int
     unsigned int i;
 
     if (fill_color.isValid()) {
-        filled = TRUE;
+        filled = true;
     } else {
         fill_color = Qt::gray;
-        filled = FALSE;
+        filled = false;
     }
 
     QBrush brush(fill_color);
@@ -1028,10 +1022,10 @@ void KeyboardLayoutWidget::drawRectangle(QPainter* painter, QColor color, int an
         bool filled;
 
         if (color.isValid()) {
-            filled = TRUE;
+            filled = true;
         } else {
             color = Qt::gray;
-            filled = FALSE;
+            filled = false;
         }
 
         x = xkbToPixmapCoord (xkb_x);
@@ -1112,26 +1106,24 @@ KeyboardLayoutWidget::~KeyboardLayoutWidget()
 
 void KeyboardLayoutWidget::drawKeyLabel(QPainter* painter, uint keycode, int angle, int xkb_origin_x, int xkb_origin_y, int xkb_width, int xkb_height, bool is_pressed)
 {
-    int x, y, width, height;
-    int padding;
-    int g, l, glp;
-
     if (!xkb)
         return;
 
-    padding = 23 * ratio;   /* 2.3mm */
+    int padding = 23 * ratio;   /* 2.3mm */
 
-    x = xkbToPixmapCoord (xkb_origin_x);
-    y = xkbToPixmapCoord (xkb_origin_y);
-    width = xkbToPixmapCoord (xkb_origin_x + xkb_width) - x;
-    height = xkbToPixmapCoord (xkb_origin_y + xkb_height) - y;
+    int x = xkbToPixmapCoord (xkb_origin_x);
+    int y = xkbToPixmapCoord (xkb_origin_y);
+    int width = xkbToPixmapCoord (xkb_origin_x + xkb_width) - x;
+    int height = xkbToPixmapCoord (xkb_origin_y + xkb_height) - y;
 
-    for (glp = KEYBOARD_DRAWING_POS_TOPLEFT;
+    QString syms[KEYBOARD_DRAWING_POS_TOTAL] = {0,0,0,0};
+
+    for (int glp = KEYBOARD_DRAWING_POS_TOPLEFT;
          glp < KEYBOARD_DRAWING_POS_TOTAL; glp++) {
         if (groupLevels[glp] == NULL)
             continue;
-        g = groupLevels[glp]->group;
-        l = groupLevels[glp]->level;
+        int g = groupLevels[glp]->group;
+        int l = groupLevels[glp]->level;
 
         if (g < 0 || g >= XkbKeyNumGroups (xkb, keycode))
             continue;
@@ -1147,42 +1139,76 @@ void KeyboardLayoutWidget::drawKeyLabel(QPainter* painter, uint keycode, int ang
                 continue;
         }
 
+        KeySym keysym = 0;
         if (trackModifiers) {
             uint mods_rtrn;
-            KeySym keysym;
 
             if (XkbTranslateKeyCode (xkb, keycode,
                          XkbBuildCoreState(mods, g),
                          &mods_rtrn, &keysym)) {
-                drawKeyLabelHelper (painter, keysym, angle, glp,
-                               x, y, width, height,
-                               padding,
-                               is_pressed);
-                /* reverse y order */
+                syms[glp] = keySymToString(keysym);
             }
         } else {
-            KeySym keysym;
+            keysym = XkbKeySymEntry (xkb, keycode, l, g);
+            syms[glp] = keySymToString(keysym);
+        }
+    }
 
-            keysym =
-                XkbKeySymEntry (xkb, keycode, l, g);
+    //  TOPLEFT     TOPRIGHT
+    //  BOTTOMLEFT  BOTTOMRIGHT
+    int end[KEYBOARD_DRAWING_POS_TOTAL];
+    const int TOPLEFT = KEYBOARD_DRAWING_POS_TOPLEFT,
+              TOPRIGHT = KEYBOARD_DRAWING_POS_TOPRIGHT,
+              BOTTOMLEFT = KEYBOARD_DRAWING_POS_BOTTOMLEFT,
+              BOTTOMRIGHT = KEYBOARD_DRAWING_POS_BOTTOMRIGHT;
+    end[TOPLEFT] = TOPLEFT;
+    end[TOPRIGHT] = TOPRIGHT;
+    end[BOTTOMLEFT] = BOTTOMLEFT;
+    end[BOTTOMRIGHT] = BOTTOMRIGHT;
 
-            drawKeyLabelHelper (painter, keysym,
-                           angle, glp, x, y, width,
-                           height, padding,
-                           is_pressed);
+    if (syms[BOTTOMLEFT] == syms[BOTTOMRIGHT] ||
+        syms[BOTTOMRIGHT].isNull()) {
+        syms[BOTTOMRIGHT] = QString::null;
+        end[BOTTOMLEFT] = BOTTOMRIGHT;
+        end[BOTTOMRIGHT] = -1;
+    }
+
+    if (syms[TOPLEFT] == syms[TOPRIGHT] ||
+        syms[TOPRIGHT].isNull()) {
+        syms[TOPRIGHT] = QString::null;
+        end[TOPLEFT] = TOPRIGHT;
+        end[TOPRIGHT] = -1;
+    }
+
+    if ((syms[BOTTOMLEFT] == syms[TOPLEFT] ||
+        syms[TOPLEFT].isNull()) &&
+        ((end[BOTTOMLEFT] == BOTTOMLEFT && end[TOPLEFT] == TOPLEFT) ||
+         (end[BOTTOMLEFT] == BOTTOMRIGHT && end[TOPLEFT] == TOPRIGHT))) {
+        syms[TOPLEFT] = QString::null;
+        end[BOTTOMLEFT] = end[TOPLEFT];
+        end[TOPLEFT] = -1;
+    }
+
+    if (!syms[BOTTOMRIGHT].isNull() && (syms[BOTTOMRIGHT] == syms[TOPRIGHT] || (syms[TOPRIGHT].isNull() && end[TOPRIGHT] != -1))) {
+        syms[TOPRIGHT] = QString::null;
+        end[BOTTOMRIGHT] = TOPRIGHT;
+    }
+
+    for (int glp = KEYBOARD_DRAWING_POS_TOPLEFT;
+         glp < KEYBOARD_DRAWING_POS_TOTAL; glp++) {
+        if (!syms[glp].isEmpty()) {
+            drawKeyLabelHelper (painter, syms[glp],
+                            angle, glp, end[glp], x, y, width,
+                            height, padding,
+                            is_pressed);
             /* reverse y order */
         }
     }
 }
 
-void KeyboardLayoutWidget::drawKeyLabelHelper(QPainter* painter, int keysym, int angle, int glp, int x, int y, int width, int height, int padding, bool is_pressed)
+void KeyboardLayoutWidget::drawKeyLabelHelper(QPainter* painter, const QString &text_, int angle, int glp, int end_glp, int x, int y, int width, int height, int padding, bool is_pressed)
 {
-    if (keysym == 0)
-        return;
-
-    if (keysym == XK_VoidSymbol)
-        return;
-
+    QString text = text_;
     if (padding >= height / 2)
         padding = 0;
     if (padding >= width / 2)
@@ -1191,54 +1217,51 @@ void KeyboardLayoutWidget::drawKeyLabelHelper(QPainter* painter, int keysym, int
     Qt::Alignment align;
 
     QRectF rect(padding, padding, (width - 2 * padding), (height - 2 * padding));
+    QRectF textRect;
+    QMarginsF margin(0, 0, 0, 0);
+    //  TOPLEFT     TOPRIGHT
+    //  BOTTOMLEFT  BOTTOMRIGHT
     switch (glp) {
     case KEYBOARD_DRAWING_POS_TOPLEFT:
         align = Qt::AlignTop | Qt::AlignLeft;
+        margin.setBottom(rect.height() / 2);
         break;
     case KEYBOARD_DRAWING_POS_BOTTOMLEFT:
         align = Qt::AlignBottom | Qt::AlignLeft;
         break;
     case KEYBOARD_DRAWING_POS_TOPRIGHT:
         align = Qt::AlignTop | Qt::AlignRight;
+        margin.setBottom(rect.height() / 2);
         break;
     case KEYBOARD_DRAWING_POS_BOTTOMRIGHT:
         align = Qt::AlignBottom | Qt::AlignRight;
+        margin.setLeft(rect.width() / 2);
         break;
     default:
         return;
     }
-
-    if (keysym == XK_ISO_Left_Tab)
-        keysym = XK_Tab;
-
-    keysym = (int) FcitxHotkeyPadToMain((FcitxKeySym) keysym);
-    uint32_t unicode = FcitxKeySymToUnicode((FcitxKeySym) keysym);
-
-    if (deadMap.contains(keysym)) {
-        unicode = deadMap[keysym];
+    switch (end_glp) {
+    case KEYBOARD_DRAWING_POS_TOPLEFT:
+        margin.setRight(rect.width() / 2);
+        break;
+    case KEYBOARD_DRAWING_POS_BOTTOMLEFT:
+        margin.setRight(rect.width() / 2);
+        margin.setTop(rect.height() / 2);
+        break;
+    case KEYBOARD_DRAWING_POS_TOPRIGHT:
+        break;
+    case KEYBOARD_DRAWING_POS_BOTTOMRIGHT:
+        margin.setTop(rect.height() / 2);
+        break;
+    default:
+        return;
     }
-    QString text;
-    if (unicode
-        && QChar(unicode).category() != QChar::Other_Control
-        && !QChar(unicode).isSpace())
-        text.append(QChar(unicode));
-    else {
-        if (keysym == XK_Prior) {
-            text = "PgUp";
-        }
-        else if (keysym == XK_Next) {
-            text = "PgDn";
-        }
-        else {
-            text = QString(XKeysymToString(keysym));
-        }
-    }
-    if (text != "_") {
-        if (text.endsWith("_L") || text.endsWith("_R"))
-            text = text.replace('_', ' ');
-        else
-            text = text.replace('_', '\n');
+    textRect = rect.marginsRemoved(margin);
+    // which means we have longer width
+    if (textRect.width() == rect.width() && textRect.height() != rect.height()) {
+        text.replace('\n', ' ');
     }
+
     painter->save();
     QTransform trans;
     trans.translate(x + padding / 2, y + padding / 2);
@@ -1251,16 +1274,33 @@ void KeyboardLayoutWidget::drawKeyLabelHelper(QPainter* painter, int keysym, int
     trans.translate(x, y);
     trans.rotate(angle / 10);
     QFont font = painter->font();
-    font.setPixelSize(TEXT_SIZE * (height / 2 - padding));
-    QFontMetricsF fm(font);
-    QStringList lines = text.split('\n');
-    foreach(const QString& line, lines) {
-        qreal w = fm.width(line);
-        if (w > (width - padding * 2)  * TEXT_SIZE) {
-            double sz = font.pixelSize() / w * (width - padding * 2) * TEXT_SIZE;
-            if (sz < 1)
-                sz = 1;
-            font.setPixelSize(font.pixelSize() / w * (width - padding * 2) * TEXT_SIZE);
+    // Normalize maximum text size
+    {
+        QFontMetricsF fm(font);
+        qreal h = fm.size(align | Qt::TextSingleLine, text).height();
+        if (h > rect.height() / 2) {
+            qreal scale = rect.height() / 2 / h;
+            font.setPointSizeF(font.pointSizeF() * scale);
+        }
+    }
+    // fit it in rect
+    {
+        QFontMetricsF fm(font);
+        qreal h = fm.size(align, text).height();
+        if (h > textRect.height()) {
+            qreal scale = textRect.height() / h;
+            font.setPointSizeF(font.pointSizeF() * scale);
+        }
+    }
+    int maxTry = 3;
+    while (maxTry--) {
+        QFontMetricsF fm(font);
+        qreal w = fm.size(align, text).width();
+        if (w > textRect.width()) {
+            qreal scale = textRect.width() / w;
+            font.setPointSizeF(font.pointSizeF() * scale);
+        } else {
+            break;
         }
     }
 
@@ -1269,11 +1309,19 @@ void KeyboardLayoutWidget::drawKeyLabelHelper(QPainter* painter, int keysym, int
 #if 0
     painter->save();
     painter->setPen(QPen(Qt::red));
-    painter->drawRect(rect);
+    painter->drawRect(textRect);
     painter->restore();
 #endif
-    painter->drawText(rect, align, text);
+    painter->drawText(textRect, align | Qt::TextDontClip, text);
     painter->restore();
+#if 0
+    QString name[KEYBOARD_DRAWING_POS_TOTAL];
+    name[KEYBOARD_DRAWING_POS_TOPLEFT] = "TOPLEFT";
+    name[KEYBOARD_DRAWING_POS_TOPRIGHT] = "TOPRIGHT";
+    name[KEYBOARD_DRAWING_POS_BOTTOMLEFT] = "BOTTOMLEFT";
+    name[KEYBOARD_DRAWING_POS_BOTTOMRIGHT] = "BOTTOMRIGHT";
+    qDebug() << "KEY" << text << name[glp] << name[end_glp] << rect << textRect;
+#endif
 }
 
 
@@ -1408,11 +1456,11 @@ void KeyboardLayoutWidget::paintEvent(QPaintEvent* event)
     QPainter p(this);
     p.setClipRect(event->rect());
     p.setCompositionMode(QPainter::CompositionMode_SourceOver);
-    int dx = (rect().width() - image.rect().width()) / 2;
-    int dy = (rect().height() - image.rect().height()) / 2;
-    QRect r = image.rect();
+    int dx = (rect().width() - image.width() / image.devicePixelRatio()) / 2;
+    int dy = (rect().height() - image.height() / image.devicePixelRatio()) / 2;
+    QRect r(QPoint(dx, dy), image.size() / image.devicePixelRatio());
     r.moveTo(dx, dy);
-    p.drawImage(r, image);
+    p.drawPixmap(r, image, image.rect());
 }
 
 void KeyboardLayoutWidget::resizeEvent(QResizeEvent* event)
@@ -1422,28 +1470,74 @@ void KeyboardLayoutWidget::resizeEvent(QResizeEvent* event)
     QWidget::resizeEvent(event);
 }
 
-bool KeyboardLayoutWidget::x11Event(XEvent* event)
+void KeyboardLayoutWidget::keyPressEvent(QKeyEvent* event)
+{
+    return keyEvent(event);
+}
+
+void KeyboardLayoutWidget::keyReleaseEvent(QKeyEvent* event)
+{
+    return keyEvent(event);
+}
+
+void KeyboardLayoutWidget::keyEvent(QKeyEvent* event)
 {
     do {
         if (!xkb)
             break;
-        if (event->type != KeyPress && event->type != KeyRelease)
+        if (event->type() != QEvent::KeyPress && event->type() != QEvent::KeyRelease)
             break;
-        DrawingKey* key = &keys[event->xkey.keycode];
-        if (event->xkey.keycode > xkb->max_key_code ||
-            event->xkey.keycode < xkb->min_key_code ||
+        DrawingKey* key = &keys[event->nativeScanCode()];
+        if (event->nativeScanCode() > xkb->max_key_code ||
+            event->nativeScanCode() < xkb->min_key_code ||
             key->xkbkey == NULL) {
             break;
         }
-        if (event->type == KeyPress && key->pressed)
+        if (event->type() == QEvent::KeyPress && key->pressed)
             break;
-        if (event->type == KeyRelease && !key->pressed)
+        if (event->type() == QEvent::KeyRelease && !key->pressed)
             break;
-        key->pressed = (event->type == KeyPress);
+        key->pressed = (event->type() == QEvent::KeyPress);
         generatePixmap(true);
         repaint();
     } while(0);
-    return QWidget::x11Event(event);
 }
 
+QString KeyboardLayoutWidget::keySymToString(unsigned long keysym) {
+    if (keysym == 0 || keysym == XK_VoidSymbol)
+        return {};
+    keysym = normalizedKeySym(keysym);
+
+    keysym = (KeySym) FcitxHotkeyPadToMain((FcitxKeySym) keysym);
+    uint32_t unicode = FcitxKeySymToUnicode((FcitxKeySym) keysym);
+
+    if (deadMap.contains(keysym)) {
+        unicode = deadMap[keysym];
+    }
+    QString text;
+    if (unicode
+        && QChar::category(unicode) != QChar::Other_Control
+        && !QChar::isSpace(unicode)) {
+        text = QString::fromUcs4(&unicode, 1);
+    }
+    else {
+        if (keysym == XK_Prior) {
+            text = "PgUp";
+        }
+        else if (keysym == XK_Next) {
+            text = "PgDn";
+        }
+        else {
+            text = QString(XKeysymToString(keysym));
+        }
+    }
+    if (text != "_") {
+        if (text.endsWith("_L") || text.endsWith("_R"))
+            text = text.replace('_', ' ');
+        else
+            text = text.replace('_', '\n');
+    }
+
+    return text;
+}
 
