@@ -5,6 +5,7 @@
 
 #include <QWidget>
 #include <QMap>
+#include <QAbstractNativeEventFilter>
 
 class QPainter;
 struct Doodad;
@@ -78,7 +79,9 @@ public:
     void generatePixmap(bool force = false);
 
 protected:
-    virtual void paintEvent(QPaintEvent* event);
+    void keyPressEvent(QKeyEvent * event) override;
+    void keyReleaseEvent(QKeyEvent * event) override;
+    void paintEvent(QPaintEvent* event) override;
     void init();
     void alloc();
     void release();
@@ -93,12 +96,12 @@ protected:
     bool parseXkbColorSpec(char* colorspec, QColor& color);
     void initColors();
 
-    virtual void focusOutEvent(QFocusEvent* event);
+    void focusOutEvent(QFocusEvent* event) override;
 
     void drawKey(QPainter* painter, DrawingKey* item);
     void drawDoodad(QPainter* painter, Doodad* doodad);
     void drawKeyLabel(QPainter* painter, uint keycode, int angle, int arg4, int originY, int x2, int y2, bool pressed);
-    void drawKeyLabelHelper(QPainter* painter, int keysym, int angle, int glp, int x, int y, int width, int height, int padding, bool is_pressed);
+    void drawKeyLabelHelper(QPainter* painter, const QString &text, int angle, int glp, int end_glp, int x, int y, int width, int height, int padding, bool is_pressed);
 
     void drawShapeDoodad(QPainter* painter, Doodad* doodad, struct _XkbShapeDoodad* shapeDoodad);
     void drawTextDoodad(QPainter* painter, Doodad* doodad, struct _XkbTextDoodad* textDoodad);
@@ -114,10 +117,13 @@ protected:
     void roundedPolygon(QPainter* painter, bool filled, double radius, const QVector< QPointF >& points);
     void drawCurveRectangle(QPainter* painter, bool filled, QColor color, int x, int y, int width, int height, double radius);
     void roundedCorner (QPainterPath& path, QPointF b, QPointF c, double radius);
-    void resizeEvent(QResizeEvent* event);
+    void resizeEvent(QResizeEvent* event) override;
     void setKeyboard(struct _XkbComponentNames* xkbDesc);
 
 private:
+    void keyEvent(QKeyEvent *keyEvent);
+    QString keySymToString(unsigned long keysym);
+
     QList<DrawingItem*> keyboardItems;
     DrawingKey* keys;
     QList<Doodad*> physicalIndicators;
@@ -126,7 +132,7 @@ private:
     int physicalIndicatorsSize;
     bool xkbOnDisplay;
     QColor* colors;
-    QImage image;
+    QPixmap image;
     double ratio;
     KeyboardDrawingGroupLevel** groupLevels;
     bool trackModifiers;
